@@ -1,4 +1,5 @@
-"Commands for the TurboGears command line tool."
+"""Commands for the TurboGears command line tool."""
+
 import optparse
 import sys
 import os
@@ -13,19 +14,21 @@ import configobj
 import turbogears
 from turbogears.util import get_model, load_project_config, \
         get_project_config, get_package_name
-from turbogears.identity import SecureObject,from_any_host
+from turbogears.identity import SecureObject, from_any_host
 from turbogears import config, database
+from sacommand import sacommand
 
 sys.path.insert(0, os.getcwd())
 
 no_connection_param = ["help", "list"]
 no_model_param = ["help"]
 
-def silent_os_remove(fname):
-    """
-    Tries to remove file FNAME but mutes any error that may happen.
 
-    Returns True if file was actually removed and false otherwise
+def silent_os_remove(fname):
+    """Try to remove file 'fname' but mute any error that may happen.
+
+    Returns True if file was actually removed and False otherwise.
+
     """
     try:
         os.remove(fname)
@@ -34,8 +37,10 @@ def silent_os_remove(fname):
         pass
     return False
 
+
 class CommandWithDB(object):
-    "Base class for commands that need to use the database"
+    """Base class for commands that need to use the database"""
+
     config = None
 
     def __init__(self, version):
@@ -45,37 +50,20 @@ class CommandWithDB(object):
         """Chooses the config file, trying to guess whether this is a
         development or installed project."""
         load_project_config(self.config)
-        self.dburi = turbogears.config.get("sqlobject.dburi", None)
+        self.dburi = config.get("sqlobject.dburi", None)
         if self.dburi and self.dburi.startswith("notrans_"):
             self.dburi = self.dburi[8:]
 
 
-[dispatch.generic()]
-def sacommand(command, args):
-    pass
-
-[sacommand.when("command == 'help'")]
-def sahelp(command, args):
-    print """TurboGears SQLAlchemy Helper
-
-help    this display
-create  create the database tables
-"""
-[sacommand.when("command == 'create'")]
-def sacreate(command, args):
-    print "Creating tables at %s" % (config.get("sqlalchemy.dburi"))
-    from turbogears.database import bind_meta_data, metadata
-    bind_meta_data()
-    get_model()
-    metadata.create_all()
-
 class SQL(CommandWithDB):
-    """
-    Wrapper command for sqlobject-admin, and provide some sqlalchemy support.
+    """Wrapper command for sqlobject-admin, and some sqlalchemy support.
 
     This automatically supplies sqlobject-admin with the database that
-    is found in the config file. Will also supply the model module as
-    appropriate."""
+    is found in the config file.
+
+    Will also supply the model module as appropriate.
+
+    """
 
     desc = "Run the database provider manager"
     need_project = True
@@ -89,23 +77,23 @@ class SQL(CommandWithDB):
                 version="%prog " + version)
             parser.add_option("-c", "--config", help="config file",
                               dest="config")
-            (options, args) = parser.parse_args(sys.argv[1:3])
+            options, args = parser.parse_args(sys.argv[1:3])
 
             if not options.config:
                 parser.error("Please provide a valid option or command.")
             self.config = options.config
             # get rid of our config option
-            if not args:
-                del sys.argv[1:3]
-            else:
+            if args:
                 del sys.argv[1]
+            else:
+                del sys.argv[1:3]
 
         self.find_config()
 
     def run(self):
-        "Executes the sqlobject-admin code."
+        """Executes the sqlobject-admin code."""
         if not "--egg" in sys.argv and not turbogears.util.get_project_name():
-            print "this don't look like a turbogears project"
+            print "This don't look like a TurboGears project."
             return
         else:
             command = sys.argv[1]
@@ -117,21 +105,27 @@ class SQL(CommandWithDB):
                     sacommand("help", [])
                 return
 
-            sqlobjcommand = command
-            if sqlobjcommand not in no_connection_param:
-                if not self.dburi:
-                    print """Database URI not specified in the config file (%s).
-        Please be sure it's on the command line.""" % self.config
-                else:
+            try:
+                from sqlobject.manager import command as socommand
+            except ImportError:
+                print "Error: SQLObject not installed."
+                return
+
+            if command not in no_connection_param:
+                if self.dburi:
                     print "Using database URI %s" % self.dburi
                     sys.argv.insert(2, self.dburi)
                     sys.argv.insert(2, "-c")
+                else:
+                    print ("Database URI not specified in the config file"
+                        " (%s).\nPlease be sure it's on the command line."
+                            % (self.config or get_project_config()))
 
-            if sqlobjcommand not in no_model_param:
+            if command not in no_model_param:
                 if not "--egg" in sys.argv:
                     eggname = glob.glob("*.egg-info")
-                    if not eggname or not \
-                        os.path.exists(os.path.join(eggname[0], "sqlobject.txt")):
+                    if not eggname or not os.path.exists(
+                            os.path.join(eggname[0], "sqlobject.txt")):
                         eggname = self.fix_egginfo(eggname)
                     eggname = eggname[0].replace(".egg-info", "")
                     if not "." in sys.path:
@@ -140,10 +134,10 @@ class SQL(CommandWithDB):
                     sys.argv.insert(2, eggname)
                     sys.argv.insert(2, "--egg")
 
-            from sqlobject.manager import command
-            command.the_runner.run(sys.argv)
+            socommand.the_runner.run(sys.argv)
 
     def fix_egginfo(self, eggname):
+        """Add egg-info directory if necessary."""
         print """
 This project seems incomplete. In order to use the sqlobject commands
 without manually specifying a model, there needs to be an
@@ -173,15 +167,18 @@ history_dir=$base/%(package)s/sqlobject-history
 
 class Shell(CommandWithDB):
     """Convenient version of the Python interactive shell.
+
     This shell attempts to locate your configuration file and model module
     so that it can import everything from your model and make it available
-    in the Python shell namespace."""
+    in the Python shell namespace.
+
+    """
 
     desc = "Start a Python prompt with your database available"
     need_project = True
 
     def run(self):
-        "Run the shell"
+        """Run the shell"""
         self.find_config()
 
         mod = get_model()
@@ -192,9 +189,9 @@ class Shell(CommandWithDB):
 
         if config.get("sqlalchemy.dburi"):
             using_sqlalchemy = True
-            database.bind_meta_data()
-            locals.update(session=database.session,
-                          metadata=database.metadata)
+            database.get_engine()
+            locals.update(dict(session=database.session,
+                metadata=database.metadata))
         else:
             using_sqlalchemy = False
 
@@ -205,13 +202,12 @@ class Shell(CommandWithDB):
             class CustomIPShell(IPython.iplib.InteractiveShell):
                 def raw_input(self, *args, **kw):
                     try:
-                        return \
-                         IPython.iplib.InteractiveShell.raw_input(self,
-                                                    *args, **kw)
+                        return IPython.iplib.InteractiveShell.raw_input(self,
+                            *args, **kw) # needs decoding (see below)?
                     except EOFError:
-                        b = raw_input("Do you wish to commit your "
+                        r = raw_input("Do you wish to commit your "
                                     "database changes? [yes]")
-                        if not b.startswith("n"):
+                        if not r.lower().startswith("n"):
                             if using_sqlalchemy:
                                 self.push("session.flush()")
                             else:
@@ -230,14 +226,21 @@ class Shell(CommandWithDB):
                         import readline
                     except ImportError:
                         pass
-
                     try:
-                        return code.InteractiveConsole.raw_input(self,
-                                                        *args, **kw)
+                        r = code.InteractiveConsole.raw_input(self,
+                            *args, **kw)
+                        for encoding in (getattr(sys.stdin, 'encoding', None),
+                                sys.getdefaultencoding(), 'utf-8', 'latin-1'):
+                            if encoding:
+                                try:
+                                    return r.decode(encoding)
+                                except UnicodeError:
+                                    pass
+                        return r
                     except EOFError:
-                        b = raw_input("Do you wish to commit your "
+                        r = raw_input("Do you wish to commit your "
                                       "database changes? [yes]")
-                        if not b.startswith("n"):
+                        if not r.lower().startswith("n"):
                             if using_sqlalchemy:
                                 self.push("session.flush()")
                             else:
@@ -247,28 +250,32 @@ class Shell(CommandWithDB):
             shell = CustomShell(locals=locals)
             shell.interact()
 
+
 class ToolboxCommand(CommandWithDB):
 
     desc = "Launch the TurboGears Toolbox"
 
     def __init__(self, version):
-        self.hostlist = ['127.0.0.1','::1']
+        self.hostlist = ['127.0.0.1', '::1', '::ffff:127.0.0.1']
 
         parser = optparse.OptionParser(
-            usage="%prog toolbox [options]", version="%prog " + version)
+            usage="%prog toolbox [options]",
+            version="%prog " + version)
         parser.add_option("-n", "--no-open",
-                 help="don't open browser automatically",
-                 dest="noopen", action="store_true",
-                 default=False)
+            help="don't open browser automatically",
+            dest="noopen", action="store_true", default=False)
         parser.add_option("-c", "--add-client",
-                help="allow the client ip address specified to connect to toolbox (Can be specified more than once)",
-                dest="host", action="append", default=None)
+            help="allow client ip address specified to connect to toolbox"
+                " (can be specified more than once)",
+            dest="host", action="append", default=None)
         parser.add_option("-p", "--port",
-                help="port to run the Toolbox on", dest="port", default=7654)
-        parser.add_option("--conf", help="config file to use",
-                dest="config", default=get_project_config())
+            help="port to run the Toolbox on",
+            dest="port", default=7654)
+        parser.add_option("--config",
+            help="config file", dest="config",
+            default=self.config or get_project_config())
 
-        (options, args) = parser.parse_args(sys.argv[1:])
+        options, args = parser.parse_args(sys.argv[1:])
         self.port = int(options.port)
         self.noopen = options.noopen
         self.config = options.config
@@ -289,15 +296,12 @@ class ToolboxCommand(CommandWithDB):
         # Make sure we have full configuration with every option
         # in it so other plugins or whatever find what they need
         # when starting even inside the toolblox
-        if get_package_name():
-            conf = turbogears.config.config_obj(
-                    configfile=self.config,
-                    modulename="%s.config" % get_package_name())
-        else:
-            conf = turbogears.config.config_obj(configfile=self.config)
+        conf = get_package_name()
+        conf = conf and "%s.config" % conf or None
+        conf = config.config_obj(configfile=self.config, modulename=conf)
 
         if 'global' in conf:
-            turbogears.config.update({'global': conf['global']})
+            config.update({'global': conf['global']})
 
         root = SecureObject(toolbox.Toolbox(), from_any_host(self.hostlist),
                 exclude=['noaccess'])
@@ -306,14 +310,20 @@ class ToolboxCommand(CommandWithDB):
 
         # amend some parameters since we are running from the command
         # line in order to change port, log methods...
-        turbogears.config.update({"global" : {
-            "server.socket_port" : self.port,
-            "server.environment" : "development",
-            "server.log_to_screen" : True,
-            "autoreload.on" : False,
-            "server.package" : "turbogears.toolbox",
-            "log_debug_info_filter.on" : False,
-            "identity.failure_url" : "/noaccess"
+        config.update({'global': {
+            'server.socket_port': self.port,
+            'server.webpath': '/',
+            'server.environment': 'development',
+            'server.log_to_screen': True,
+            'autoreload.on': False,
+            'server.package': 'turbogears.toolbox',
+            'log_debug_info_filter.on': False,
+            'identity.failure_url': '/noaccess',
+            'identity.force_external_redirect': False,
+            'tg.defaultview': 'kid',
+            'tg.strict_parameters': False,
+            'kid.outputformat': 'html default',
+            'kid.encoding': 'utf-8'
             }})
 
         if not self.noopen:
@@ -321,10 +331,11 @@ class ToolboxCommand(CommandWithDB):
         else:
             cherrypy.server.start()
 
+
 commands = None
 
 def main():
-    "Main command runner. Manages the primary command line arguments."
+    """Main command runner. Manages the primary command line arguments."""
     # add commands defined by entrypoints
     commands = {}
     for entrypoint in pkg_resources.iter_entry_points("turbogears.command"):
@@ -332,7 +343,7 @@ def main():
         commands[entrypoint.name] = (command.desc, entrypoint)
 
     def _help():
-        "Custom help text for tg-admin."
+        """Custom help text for tg-admin."""
 
         print """
 TurboGears %s command line interface
@@ -352,22 +363,21 @@ Commands:""" % (turbogears.__version__, sys.argv[0])
         for key in commandlist:
             print format % (key, commands[key][0])
 
-
     parser = optparse.OptionParser()
     parser.allow_interspersed_args = False
     parser.add_option("-c", "--config", dest="config")
     parser.add_option("-e", "--egg", dest="egg")
     parser.print_help = _help
-    (options, args) = parser.parse_args(sys.argv[1:])
+    options, args = parser.parse_args(sys.argv[1:])
 
-    # if not command is found display help
+    # if command is not found display help
     if not args or not commands.has_key(args[0]):
         _help()
         sys.exit()
 
     commandname = args[0]
     # strip command and any global options from the sys.argv
-    sys.argv = [sys.argv[0],] + args[1:]
+    sys.argv = [sys.argv[0]] + args[1:]
     command = commands[commandname][1]
     command = command.load()
 
@@ -388,4 +398,5 @@ tg-admin --config=<FILE> %s""" % commandname
     command = command(turbogears.__version__)
     command.run()
 
+
 __all__ = ["main"]
