@@ -1,36 +1,41 @@
 """Classes and methods for TurboGears controllers."""
+
 import logging
 import re
 import urllib
+import urlparse
 import types
 from itertools import izip
-import cherrypy
 from dispatch import generic, strategy, functions
-import turbogears.util as tg_util
-import turbogears 
 from inspect import isclass
+import cherrypy
+from cherrypy import request, response
+import turbogears.util as tg_util
 from turbogears import view, database, errorhandling, config
 from turbogears.decorator import weak_signature_decorator
 from turbogears.validators import Invalid
 from turbogears.errorhandling import error_handler, exception_handler
 
+
 log = logging.getLogger("turbogears.controllers")
 
-unicodechars = re.compile(r"([^\x00-\x7F])")
 
-if config.get("session_filter.on",None) == True:
-    if config.get("session_filter.storage_type",None) == "PostgreSQL":
-        import psycopg2 
-        config.update({'session_filter.get_db':psycopg2.connect(psycopg2.get('sessions.postgres.dsn'))})
+if config.get("session_filter.on", None):
+    if config.get("session_filter.storage_type", None) == "PostgreSQL":
+        import psycopg2
+        config.update(
+                {'session_filter.get_db': psycopg2.connect(
+                    psycopg2.get('sessions.postgres.dsn'))
+                    })
     # support for mysql/sqlite/etc here
 
-def _process_output(output, template, format, content_type, mapping, fragment=False):
-    """Produces final output form from the data returned from a
-    controller method.
 
-    @param tg_format: format of desired output (html or json)
-    @param output: the output returned by the controller
-    @param template: HTML template to use
+def _process_output(output, template, format, content_type,
+        mapping, fragment=False):
+    """Produce final output form from data returned from a controller method.
+
+    See the expose() arguments for more info since they are the same.
+
     """
     if isinstance(output, dict):
         from turbogears.widgets import js_location
@@ -42,9 +47,9 @@ def _process_output(output, template, format, content_type, mapping, fragment=Fa
 
         if config.get("tg.mochikit_all", False):
             include_widgets_lst.insert(0, 'turbogears.mochikit')
-            
+
         for i in include_widgets_lst:
-            widget = tg_util.load_class(i)  
+            widget = tg_util.load_class(i)
             if isclass(widget):
                 widget = widget()
             include_widgets["tg_%s" % i.split(".")[-1]] = widget
@@ -70,64 +75,82 @@ def _process_output(output, template, format, content_type, mapping, fragment=Fa
                             js[js_location.head].add(script)
         output.update(include_widgets)
         output["tg_css"] = css
-        #output.update([("tg_js_%s" % str(l), js[l]) for l in js_location])
-        for l in iter(js_location):
-            output["tg_js_%s" % str(l)] = js[l]
+        for location in iter(js_location):
+            output["tg_js_%s" % str(location)] = js[location]
 
-        tg_flash = _get_flash() 
-        if not tg_flash == None:
-            output["tg_flash"] = tg_flash 
+        tg_flash = _get_flash()
+        if tg_flash:
+            output["tg_flash"] = tg_flash
+
+        headers = {'Content-Type': content_type}
         output = view.render(output, template=template, format=format,
-                    mapping=mapping, content_type=content_type,
+                    mapping=mapping, headers=headers,
                     fragment=fragment)
+        content_type = headers['Content-Type']
+
+    if content_type:
+        response.headers["Content-Type"] = content_type
     else:
-        if content_type:
-            cherrypy.response.headers["Content-Type"] = content_type
+        content_type = response.headers.get("Content-Type", 'text/plain')
+
+    if content_type.startswith("text/"):
+        if isinstance(output, unicode):
+            output = output.encode(tg_util.get_template_encoding_default())
 
-    # fix the Safari XMLHttpRequest encoding problem
-    try:
-        contentType = cherrypy.response.headers["Content-Type"]
-        ua = cherrypy.request.headers["User-Agent"]
-    except KeyError:
-        return output
-    if not contentType.startswith("text/"):
-        return output
-    ua = view.UserAgent(ua)
-    enc = tg_util.get_template_encoding_default()
-    if ua.browser == "safari":
-        if isinstance(output, str):
-            output = output.decode(enc)
-        output = unicodechars.sub(
-            lambda m: "&#x%x;" % ord(m.group(1)), output).encode("ascii")
-    if isinstance(output, unicode):
-        output = output.encode(enc)
     return output
 
+
 class BadFormatError(Exception):
     """Output-format exception."""
 
+
 def validate(form=None, validators=None,
              failsafe_schema=errorhandling.FailsafeSchema.none,
              failsafe_values=None, state_factory=None):
     """Validate input.
 
-    @param form form to validate input from
-    @param validators individual validators to use for parameters
-    @param failsafe_schema fail-safe schema
-    @param failsafe_values replacements for erroneous inputs
-    @param state_factory callable which returns the initial state instance for
-           validation
+    @param form: a form instance that must be passed throught the validation
+    process... you must give a the same form instance as the one that will
+    be used to post data on the controller you are putting the validate
+    decorator on.
+    @type form: a form instance
+
+    @param validators: individual validators to use for parameters.
+    If you use a schema for validation then the schema instance must
+    be the sole argument.
+    If you use simple validators, then you must pass a dictionary with
+    each value name to validate as a key of the dictionary and the validator
+    instance (eg: tg.validators.Int() for integer) as the value.
+    @type validators: dictionary or schema instance
+
+    @param failsafe_schema: a schema for handling failsafe values.
+    The default is 'none', but you can also use 'values', 'map_errors',
+    or 'defaults' to map erroneous inputs to values, corresponding exceptions
+    or method defaults.
+    @type failsafe_schema: errorhandling.FailsafeSchema
+
+    @param failsafe_values: replacements for erroneous inputs. You can either
+    define replacements for every parameter, or a single replacement value
+    for all parameters. This is only used when failsafe_schema is 'values'.
+    @type failsafe_values: a dictionary or a single value
+
+    @param state_factory: If this is None, the initial state for validation
+    is set to None, otherwise this must be a callable that returns the initial
+    state to be used for validation.
+    @type state_factory: callable or None
+
     """
     def entangle(func):
-        recursion_guard = dict(func=func)
         if callable(form) and not hasattr(form, "validate"):
             init_form = lambda self: form(self)
         else:
             init_form = lambda self: form
 
         def validate(func, *args, **kw):
-            if tg_util.call_on_stack("validate", recursion_guard, 4):
+            # do not validate a second time if already validated
+            if hasattr(request, 'validation_state'):
                 return func(*args, **kw)
+
             form = init_form(args and args[0] or kw["self"])
             args, kw = tg_util.to_kw(func, args, kw)
 
@@ -143,16 +166,15 @@ def validate(form=None, validators=None,
                     kw.update(form.validate(value, state))
                 except Invalid, e:
                     errors = e.unpack_errors()
-                    cherrypy.request.validation_exception = e
-                cherrypy.request.validated_form = form
+                    request.validation_exception = e
+                request.validated_form = form
 
             if validators:
                 if isinstance(validators, dict):
                     for field, validator in validators.iteritems():
                         try:
                             kw[field] = validator.to_python(
-                                kw.get(field, None), state
-                                )
+                                kw.get(field, None), state)
                         except Invalid, error:
                             errors[field] = error
                 else:
@@ -161,10 +183,10 @@ def validate(form=None, validators=None,
                         kw.update(validators.to_python(value, state))
                     except Invalid, e:
                         errors = e.unpack_errors()
-                        cherrypy.request.validation_exception = e
-            cherrypy.request.validation_errors = errors
-            cherrypy.request.input_values = kw.copy()
-            cherrypy.request.validation_state = state
+                        request.validation_exception = e
+            request.validation_errors = errors
+            request.input_values = kw.copy()
+            request.validation_state = state
 
             if errors:
                 kw = errorhandling.dispatch_failsafe(failsafe_schema,
@@ -175,21 +197,23 @@ def validate(form=None, validators=None,
         return validate
     return weak_signature_decorator(entangle)
 
+
 class CustomDispatch(functions.GenericFunction):
 
-    def combine(self,cases):
-        strict = [strategy.ordered_signatures,strategy.safe_methods]
+    def combine(self, cases):
+        strict = [strategy.ordered_signatures, strategy.safe_methods]
         cases = strategy.separate_qualifiers(
             cases,
             primary = strict,
         )
-        primary = strategy.method_chain(cases.get('primary',[]))
+        primary = strategy.method_chain(cases.get('primary', []))
         if type(primary) != types.FunctionType:
             for i in primary:
                 for y in i:
                     return y[1]
         return primary
 
+
 def _add_rule(_expose, found_default, as_format, accept_format, template,
               rulefunc):
     if as_format == "default":
@@ -201,34 +225,31 @@ def _add_rule(_expose, found_default, as_format, accept_format, template,
                 as_format = template[:colon]
         else:
             found_default = True
-    ruleparts = []
-    ruleparts.append('kw.get("tg_format", "default") == "%s"'
-                % as_format)
+    ruleparts = ['kw.get("tg_format", "default") == "%s"' % as_format]
     if accept_format:
-        ruleparts.append('(accept == "%s" and kw.get("tg_format", '
-                '"default") == "default")' % accept_format)
+        ruleparts.append('(accept == "%s" and kw.get("tg_format"'
+            ', "default") == "default")' % accept_format)
     rule = " or ".join(ruleparts)
     log.debug("Generated rule %s", rule)
     _expose.when(rule)(rulefunc)
 
     return found_default
 
+
 def _build_rules(func):
+    [generic(CustomDispatch)]
     def _expose(func, accept, allow_json, *args, **kw):
         pass
-    _expose = generic(CustomDispatch)(_expose)
 
     if func._allow_json:
         log.debug("Adding allow_json rule: "
-            'allow_json and '
-            '(kw.get("tg_format", None) == "json" or accept '
-            '=="text/javascript")')
-        _expose.when('allow_json '
-            'and (kw.get("tg_format", None) == "json" or accept'
-            ' =="text/javascript")')(
-            lambda _func, accept, allow_json,
-                *args, **kw: _execute_func(
-                    _func, "json", None, None, None, False, args, kw))
+            'allow_json and (kw.get("tg_format", None) == "json"'
+            ' or accept in ("application/json", "text/javascript"))')
+        _expose.when('allow_json and (kw.get("tg_format", None) == "json"'
+            ' or accept in ("application/json", "text/javascript"))')(
+            lambda _func, accept, allow_json, *args, **kw:
+                _execute_func(_func, "json", "json", "application/json",
+                    None, False, args, kw))
 
     found_default = False
     for ruleinfo in func._ruleinfo:
@@ -236,6 +257,7 @@ def _build_rules(func):
 
     func._expose = _expose
 
+
 def expose(template=None, validators=None, allow_json=None, html=None,
            format=None, content_type=None, inputform=None, fragment=False,
            as_format="default", mapping=None, accept_format=None):
@@ -256,13 +278,13 @@ def expose(template=None, validators=None, allow_json=None, html=None,
     the ":" in the template name (or the whole template name if there
     is no ":". For example, <code>expose("json")</code>, if it's not
     the default expose, will have as_format set to "json".
-    
+
     When as_format is set, passing the same value in the tg_format
     parameter in a request will choose the options for that expose
     decorator. Similarly, accept_format will watch for matching
     Accept headers. You can also use both. expose("json", as_format="json",
-    accept_format="text/javascript") will choose JSON output for either
-    case: tg_format=json as a parameter or Accept: text/javascript as a
+    accept_format="application/json") will choose JSON output for either
+    case: tg_format=json as a parameter or Accept: application/json as a
     request header.
 
     Passing allow_json=True to an expose decorator
@@ -296,25 +318,26 @@ def expose(template=None, validators=None, allow_json=None, html=None,
             engine
     @keyparam as_format designates which value of tg_format will choose
             this expose.
-    @keyparam accept_format which value of an Accept: header will 
+    @keyparam accept_format which value of an Accept: header will
             choose this expose.
     @keyparam html deprecated in favor of template
     @keyparam validators deprecated. Maps argument names to validator
             applied to that arg
-    @keyparam inputform deprecated. A form object that generates the 
+    @keyparam inputform deprecated. A form object that generates the
             input to this method
+
     """
     if html:
         template = html
     if not template:
         template = format
-    if format == "json" or (format == None and template == None):
+    if format == "json" or (format is None and template is None):
         template = "json"
         allow_json = True
     if content_type is None:
         content_type = config.get("tg.content_type", None)
 
-    if config.get("tg.session.automatic_lock",None) == True:
+    if config.get("tg.session.automatic_lock", None):
         cherrypy.session.acquire_lock()
 
     def entangle(func):
@@ -323,32 +346,31 @@ def expose(template=None, validators=None, allow_json=None, html=None,
             "content-type: %s", template, format, allow_json, content_type)
         if not getattr(func, "exposed", False):
             def expose(func, *args, **kw):
-                accept = cherrypy.request.headers.get('Accept', "").lower()
+                accept = request.headers.get('Accept', "").lower()
+                accept = tg_util.simplify_http_accept_header(accept)
                 if not hasattr(func, "_expose"):
                     _build_rules(func)
-                if hasattr(cherrypy.request, "in_transaction"):
+                if hasattr(request, "in_transaction"):
                     output = func._expose(func, accept, func._allow_json,
                                 *args, **kw)
                 else:
-                    cherrypy.request.in_transaction = True
+                    request.in_transaction = True
                     output = database.run_with_transaction(
                             func._expose, func, accept, func._allow_json,
                             *args, **kw)
                 return output
             func.exposed = True
             func._ruleinfo = []
-            allow_json_from_config = config.get(
-                                        "tg.allow_json", False)
-            func._allow_json = allow_json_from_config
+            allow_json_from_config = config.get("tg.allow_json", False)
+            func._allow_json = allow_json_from_config or template == 'json'
         else:
             expose = lambda func, *args, **kw: func(*args, **kw)
 
-        func._ruleinfo.insert(0, dict(as_format = as_format,
-            accept_format = accept_format, template = template,
-            rulefunc = lambda _func, accept, allow_json,
-                    *args, **kw:
-                    _execute_func(_func, template, format, content_type,
-                                mapping, fragment, args, kw)))
+        func._ruleinfo.insert(0, dict(as_format=as_format,
+            accept_format=accept_format, template=template,
+            rulefunc = lambda _func, accept, allow_json, *args, **kw:
+                _execute_func(_func, template, format, content_type,
+                    mapping, fragment, args, kw)))
 
         if allow_json:
             func._allow_json = True
@@ -365,40 +387,50 @@ def expose(template=None, validators=None, allow_json=None, html=None,
         return expose
     return weak_signature_decorator(entangle)
 
+
 def _execute_func(func, template, format, content_type, mapping, fragment, args, kw):
     """Call controller method and process it's output."""
     if config.get("tg.strict_parameters", False):
-	    tg_util.remove_keys(kw, ["tg_random", "tg_format"])
+        tg_util.remove_keys(kw, ["tg_random", "tg_format"]
+            + config.get("tg.ignore_parameters", []))
     else:
+        # get special parameters used by upstream decorators like paginate
+        try:
+            tg_kw = dict([(k, v) for k, v in kw.items() if k in func._tg_args])
+        except AttributeError:
+            tg_kw = {}
+        # remove excessive parameters
         args, kw = tg_util.adapt_call(func, args, kw)
+        # add special parameters again
+        kw.update(tg_kw)
     if config.get('server.environment', 'development') == 'development':
         # Only output this in development mode: If it's a field storage object,
         # this means big memory usage, and we don't want that in production
         log.debug("Calling %s with *(%s), **(%s)", func, args, kw)
     output = errorhandling.try_call(func, *args, **kw)
-    if isinstance(output, list):
-        return output
-    assert isinstance(output, basestring) or isinstance(output, dict) \
-        or isinstance(output, types.GeneratorType), \
+    assert isinstance(output, (basestring, dict, list, types.GeneratorType)), \
            "Method %s.%s() returned unexpected output. Output should " \
-           "be of type basestring, dict or generator." % (
+           "be of type basestring, dict, list or generator." % (
             args[0].__class__.__name__, func.__name__)
     if isinstance(output, dict):
         template = output.pop("tg_template", template)
-        format= output.pop("tg_format", format)
+        format = output.pop("tg_format", format)
     if template and template.startswith("."):
         template = func.__module__[:func.__module__.rfind('.')]+template
     return _process_output(output, template, format, content_type, mapping, fragment)
 
+
 def flash(message):
     """Set a message to be displayed in the browser on next page display."""
-    cherrypy.response.simple_cookie['tg_flash'] = tg_util.to_utf8(message)
-    cherrypy.response.simple_cookie['tg_flash']['path'] = '/'
+    message = tg_util.quote_cookie(tg_util.to_utf8(message))
+    response.simple_cookie['tg_flash'] = message
+    response.simple_cookie['tg_flash']['path'] = '/'
+
 
 def _get_flash():
     """Retrieve the flash message (if one is set), clearing the message."""
-    request_cookie = cherrypy.request.simple_cookie
-    response_cookie = cherrypy.response.simple_cookie
+    request_cookie = request.simple_cookie
+    response_cookie = response.simple_cookie
 
     def clearcookie():
         response_cookie["tg_flash"] = ""
@@ -418,21 +450,25 @@ def _get_flash():
     else:
         message = None
     if message:
-        message = unicode(message, 'utf-8')
+        message = unicode(tg_util.unquote_cookie(message), 'utf-8')
     return message
 
+
 class Controller(object):
     """Base class for a web application's controller.
 
     Currently, this provides positional parameters functionality
     via a standard default method.
+
     """
 
+
 class RootController(Controller):
     """Base class for the root of a web application.
 
     Your web application should have one of these. The root of
     your application is used to compute URLs used by your app.
+
     """
     is_app_root = True
 
@@ -447,67 +483,95 @@ class RootController(Controller):
     def _cp_log_access(self):
         tmpl = '%(h)s %(l)s %(u)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
         try:
-            username = cherrypy.request.user_name
-            if not username:
+            username = request.user_name
+            if username:
+                if isinstance(username, unicode):
+                    # might get concatenated to non-ascii byte strings
+                    username = username.encode('utf-8')
+            else:
                 username = "-"
         except AttributeError:
             username = "-"
-        s = tmpl % {'h': cherrypy.request.remote_host,
+        s = tmpl % {'h': request.remote_host,
                    'l': '-',
                    'u': username,
-                   'r': cherrypy.request.requestLine,
-                   's': cherrypy.response.status.split(" ", 1)[0],
-                   'b': cherrypy.response.headers.get('Content-Length',
+                   'r': request.requestLine,
+                   's': response.status.split(" ", 1)[0],
+                   'b': response.headers.get('Content-Length',
                             '') or "-",
-                   'f': cherrypy.request.headers.get('referer', ''),
-                   'a': cherrypy.request.headers.get('user-agent', ''),
+                   'f': request.headers.get('referer', ''),
+                   'a': request.headers.get('user-agent', ''),
         }
         self.accesslog.info(s)
 
 Root = RootController
 
+
 def url(tgpath, tgparams=None, **kw):
     """Computes URLs.
 
     tgpath can be a list or a string. If the path is absolute (starts
-    with a "/"), the server.webpath and the approot of the application
-    are prepended to the path. In order for the approot to be
-    detected properly, the root object should extend
+    with a "/"), the server.webpath, SCRIPT_NAME and the approot of the
+    application are prepended to the path. In order for the approot to
+    be detected properly, the root object should extend
     controllers.RootController.
 
     Query parameters for the URL can be passed in as a dictionary in
     the second argument *or* as keyword parameters.
+
+    Values which are a list or a tuple are used to create multiple
+    key-value pairs.
+
     """
     if not isinstance(tgpath, basestring):
-        tgpath = "/".join(list(tgpath))
-    if tgpath.startswith("/"):
+        tgpath = '/'.join(list(tgpath))
+    if tgpath.startswith('/'):
+        webpath = (config.get('server.webpath') or '').rstrip('/')
         if tg_util.request_available():
             check_app_root()
-            tgpath = cherrypy.request.app_root + tgpath
-        result = config.get("server.webpath", "") + tgpath
-    else:
-        result = tgpath
-    if tgparams is not None:
-        tgparams.update(kw)
-    else:
+            tgpath = request.app_root + tgpath
+            try:
+                webpath += request.wsgi_environ['SCRIPT_NAME'].rstrip('/')
+            except (AttributeError, KeyError):
+                pass
+        tgpath = webpath + tgpath
+    if tgparams is None:
         tgparams = kw
+    else:
+        try:
+            tgparams = tgparams.copy()
+            tgparams.update(kw)
+        except AttributeError:
+            raise TypeError('url() expects a dictionary for query parameters')
     args = []
     for key, value in tgparams.iteritems():
         if value is None:
             continue
-        if isinstance(value, unicode):
-            value = value.encode("utf8")
-        args.append("%s=%s" % (key, urllib.quote(str(value))))
+        if isinstance(value, (list, tuple)):
+            pairs = [(key, v) for v in value]
+        else:
+            pairs = [(key, value)]
+        for k, v in pairs:
+            if v is None:
+                continue
+            if isinstance(v, unicode):
+                v = v.encode('utf8')
+            args.append((k, str(v)))
     if args:
-        result += "?" + "&".join(args)
-    return result
+        query_string = urllib.urlencode(args, True)
+        if '?' in tgpath:
+            tgpath += '&' + query_string
+        else:
+            tgpath += '?' + query_string
+    return tgpath
+
 
 def check_app_root():
-    """Sets cherrypy.request.app_root if needed."""
-    if hasattr(cherrypy.request, "app_root"):
+    """Sets request.app_root if needed."""
+    if hasattr(request, 'app_root'):
         return
     found_root = False
-    trail = cherrypy.request.object_trail
+    trail = request.object_trail
     top = len(trail) - 1
     # compute the app_root by stepping back through the object
     # trail and collecting up the path elements after the first
@@ -524,23 +588,42 @@ def check_app_root():
             found_root = True
         if found_root and i > 0:
             rootlist.insert(0, path)
-    app_root = "/".join(rootlist)
-    if not app_root.startswith("/"):
-        app_root = "/" + app_root
-    if app_root.endswith("/"):
+    app_root = '/'.join(rootlist)
+    if not app_root.startswith('/'):
+        app_root = '/' + app_root
+    if app_root.endswith('/'):
         app_root = app_root[:-1]
-    cherrypy.request.app_root = app_root
+    request.app_root = app_root
+
 
 def redirect(redirect_path, redirect_params=None, **kw):
-    """
-    Redirect (via cherrypy.HTTPRedirect).
-    Raises the exception instead of returning it, this to allow
-    users to both call it as a function or to raise it as an exeption.
-    """
-    raise cherrypy.HTTPRedirect(
-                    url(tgpath=redirect_path, tgparams=redirect_params, **kw))
+    """Redirect (via cherrypy.HTTPRedirect).
 
-__all__ = ["expose", "validate", "redirect", "flash",
-           "Root", "RootController", "Controller",
-           "error_handler", "exception_handler",
-          ]
+    Raises the exception instead of returning it, this to allow
+    users to both call it as a function or to raise it as an exception.
+
+    """
+    if not isinstance(redirect_path, basestring):
+        redirect_path = '/'.join(list(redirect_path))
+    if not redirect_path.startswith('/'):
+        path = request.path_info
+        check_app_root()
+        if path.startswith(request.app_root):
+            path = path[len(request.app_root):]
+        redirect_path = urlparse.urljoin(path, redirect_path)
+    raise cherrypy.HTTPRedirect(url(tgpath=redirect_path,
+        tgparams=redirect_params, **kw))
+
+
+__all__ = [
+    "Controller",
+    "error_handler",
+    "exception_handler",
+    "expose",
+    "flash",
+    "redirect",
+    "Root",
+    "RootController",
+    "url",
+    "validate",
+]
