@@ -18,10 +18,8 @@ class SqlAlchemyVisitManager(BaseVisitManager):
 
     def __init__(self, timeout):
         global visit_class
-        super(SqlAlchemyVisitManager, self).__init__(timeout)
         visit_class_path = config.get("visit.saprovider.model",
             "turbogears.visit.savisit.TG_Visit")
-
         visit_class = load_class(visit_class_path)
         if visit_class is None:
             msg = 'No visit class found for %s' % visit_class_path
@@ -31,6 +29,9 @@ class SqlAlchemyVisitManager(BaseVisitManager):
         get_engine()
         if visit_class is TG_Visit:
             mapper(visit_class, visits_table)
+        # base-class' __init__ triggers self.create_model, so mappers need to
+        # be initialized before.
+        super(SqlAlchemyVisitManager, self).__init__(timeout)
 
     def create_model(self):
         """Create the Visit table if it doesn't already exist."""
@@ -52,15 +53,18 @@ class SqlAlchemyVisitManager(BaseVisitManager):
         Returns None if the visit doesn't exist or has expired.
 
         """
-        visit = visit_class.lookup_visit(visit_key)
-        if not visit:
+        try:
+            expiry = self.queue[visit_key]
+        except KeyError:
+            visit = visit_class.lookup_visit(visit_key)
+            if not visit:
+                return None
+            expiry = visit.expiry
+        now = datetime.now(expiry.tzinfo)
+        if expiry < now:
             return None
-        now = datetime.now(visit.expiry.tzinfo)
-        if visit.expiry < now:
-            return None
-
         # Visit hasn't expired, extend it
-        self.update_visit(visit_key, now+self.timeout)
+        self.update_visit(visit_key, now + self.timeout)
         return Visit(visit_key, False)
 
     def update_queued_visits(self, queue):
@@ -70,8 +74,8 @@ class SqlAlchemyVisitManager(BaseVisitManager):
         for visit_key, expiry in queue.items():
             log.info("updating visit (%s) to expire at %s", visit_key,
                       expiry)
-            get_engine().execute(table.update(table.c.visit_key==visit_key,
-                              values={'expiry': expiry}))
+            get_engine().execute(table.update(table.c.visit_key == visit_key,
+                values=dict(expiry=expiry)))
 
 
 # The Visit table
