@@ -19,6 +19,7 @@ __all__ = ["InputWidget", "CompoundInputWidget", "RepeatingInputWidget",
            "RepeatingFieldSet", "Form", "TableForm", "ListForm",
            "WidgetsList"]
 
+
 #############################################################################
 # Functions and classes to manage input widgets                             #
 #############################################################################
@@ -130,6 +131,7 @@ def build_name_from_path(path, repeating_marker='-', nesting_marker='.'):
             name.append(item.widget.name)
     return nesting_marker.join(name)
 
+
 ###############################################################################
 # Base class for a widget that can generate input for the app.
 ###############################################################################
@@ -142,17 +144,18 @@ class InputWidget(Widget):
     convert = True
 
     def __init__(self, name=None, validator=None, **params):
-        """Initializes an input widget.
+        """Initialize an input widget.
 
         It accepts the following parameters (besides those listed at params):
 
-          name:
-             Name of the input element. Will be the name of the variable
-             the widget's input will be available at when submitted.
+        name:
+            Name of the input element. Will be the name of the variable
+            the widget's input will be available at when submitted.
+
+        validator:
+            Formencode validator that will validate and coerce the input
+            this widget generates.
 
-          validator:
-             Formencode validator that will validate and coerce the input
-             this widget generates.
         """
         if name is not None and ('-' in name or '.' in name):
             raise ValueError, ("The name of an InputWidget must not contain "
@@ -254,6 +257,7 @@ class InputWidget(Widget):
         errors = getattr(request, "validation_errors", {})
         d["error"] = retrieve_value_by_path(errors, self.name_path)
 
+
 class CompoundInputWidget(CompoundWidget, InputWidget):
 
     def update_params(self, params):
@@ -261,23 +265,32 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
         params['error_for'] = lambda f: self.error_for(f, True)
 
     def value_for(self, item, value):
-        """Return the value for a child widget. ``item`` is the child
-        widget instance or it's name, ``value`` is a dict containing the
-        value for this compound widget."""
+        """Return the value for a child widget.
+
+        ``item`` is the child widget instance or it's name, ``value`` is a
+        dict containing the value for this compound widget.
+
+        """
         path = path_from_item(item)
         return retrieve_value_by_path(value, path)
 
     def params_for(self, item, **params):
-        """Return the parameters for a child widget. ``item`` is the child
-        widget instance or it's name, ``params`` is a dict containing the
-        parameters passed to this compound widget."""
+        """Return the parameters for a child widget.
+
+        ``item`` is the child widget instance or it's name, ``params`` is a
+        dict containing the parameters passed to this compound widget.
+
+        """
         path = path_from_item(item)
         return retrieve_params_by_path(params, path)
 
     def error_for(self, item, suppress_errors=False):
-        """Return's the Invalid exception associated to a child widget from
-        request local storage. ``item`` is the child widget instance or it's
-        name."""
+        """Return the Invalid exception associated with a child widget.
+
+        The exception is stored in the request local storage. ``item`` is the
+        child widget instance or it's name.
+
+        """
         if self.is_validated:
             path = path_from_item(item, self.name_path)
             errors = getattr(request, "validation_errors", {})
@@ -291,10 +304,14 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
             return None
 
     def dictify_value(self, value):
-        """Converts a value into a dict suitable for propagating values to
-        child widgets. If value is a dict it will pass thorugh, if it other
-        kind of object, attributes which match child widgets' names will be
-        tried to fetch."""
+        """Convert value into a dict suitable for propagating values to child
+        widgets.
+
+        If value is a dict it will pass through, if it's another kind of
+        object, attributes which match child widgets' names will tried to be
+        fetched.
+
+        """
         if isinstance(value, dict):
             return value
         else:
@@ -312,6 +329,7 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
             value = self.dictify_value(value)
         return super(CompoundWidget, self).adjust_value(value, **params)
 
+
 class RepeatingRange(object):
 
     def __init__(self, repetitions, bunch):
@@ -332,6 +350,7 @@ class RepeatingRange(object):
             self.__bunch.repetition = value
             return value
 
+
 class RepeatingInputWidget(CompoundInputWidget):
     """Base class for a compound widget which can be repeated."""
 
@@ -356,9 +375,12 @@ class RepeatingInputWidget(CompoundInputWidget):
         d["repetitions"] = RepeatingRange(repetitions, path_reference)
 
     def value_for(self, item, value):
-        """Return the value for a child widget. ``item`` is the child
-        widget instance or it's name, ``value`` is a dict containing the
-        value for this compound widget."""
+        """Return the value for a child widget.
+
+        ``item`` is the child widget instance or it's name, ``value`` is a dict
+        containing the value for this compound widget.
+
+        """
         if isinstance(value, list):
             try:
                 value = value[self.path[-1].repetition]
@@ -370,9 +392,12 @@ class RepeatingInputWidget(CompoundInputWidget):
         return retrieve_value_by_path(value, path)
 
     def params_for(self, item, **params):
-        """Return the parameters for a child widget. ``item`` is the child
-        widget instance or it's name, ``params`` is a dict containing the
-        parameters passed to this compound widget."""
+        """Return the parameters for a child widget.
+
+        ``item`` is the child widget instance or it's name, ``params`` is a
+        dict containing the parameters passed to this compound widget.
+
+        """
         item_params = {}
         for k,v in params.iteritems():
             if isinstance(v, list) and k is not "repetitions":
@@ -384,14 +409,18 @@ class RepeatingInputWidget(CompoundInputWidget):
         return retrieve_params_by_path(item_params, path)
 
     def dictify_value(self, value):
-        """Converts a list of values into a list of dicts suitable for
-        propagating values to child widgets. If value is a list of dicts it
-        will pass thorugh, if it other kind of object, attributes which match
-        child widgets' names will be
-        tried to fetch."""
+        """Convert list of values into a list of dicts suitable for propagating
+        values to child widgets.
+
+        If value is a list of dicts it will pass through, if it's another kind
+        of object, attributes which match child widgets' names will tried to be
+        fetched.
+
+        """
         return [super(RepeatingInputWidget, self).dictify_value(v)
                 for v in value]
 
+
 #############################################################################
 # Base classes                                                              #
 #############################################################################
@@ -409,7 +438,6 @@ class FormField(InputWidget):
        widget.
 
     """
-
     _name = "widget"
     label = None
     help_text = None
@@ -466,6 +494,7 @@ class FormField(InputWidget):
         d["help_text"] = self.help_text
         d["field_id"] = self.field_id
 
+
 # A decorator that provides field_for functionality to the
 # decorated FormFieldsContainer's method
 def retrieve_field_for(func):
@@ -486,6 +515,7 @@ def retrieve_field_for(func):
     _retrieve_field_for = update_path(_retrieve_field_for)
     return _retrieve_field_for
 
+
 class FormFieldsContainer(CompoundInputWidget):
     """A container for FormFields.
 
@@ -495,22 +525,21 @@ class FormFieldsContainer(CompoundInputWidget):
 
     It provides the template with 3 useful functions:
        - field_for(name)
-           Returns the child named ``name``
+           Returns the child named ``name``.
        - value_for(name)
            Returns the value for the child named ``name`` (name can also be
-           a widget instance so fields can be iterated)
+           a widget instance so fields can be iterated).
        - params_for(name)
-           Returns the display-time parameters for the child named ``name`` (can
-           also be a widget instance so fields can be iterated)
+           Returns the display-time parameters for the child named ``name``
+           (can also be a widget instance so fields can be iterated).
        - display_field_for(name)
-           displays the child named ``name`` automatically propagating value
+           Displays the child named ``name`` automatically propagating value
            an params. ``name`` can also be a widget instance.
        - error_for(name)
            Returns the validation error the child named ``name`` generated.
            Again, ``name`` can also be a widget instance.
 
     """
-
     member_widgets = ["fields", "hidden_fields"]
     fields = []
     hidden_fields = []
@@ -539,9 +568,13 @@ class FormFieldsContainer(CompoundInputWidget):
     render_field_for = retrieve_field_for(render_field_for)
 
     def _field_for(self, item):
-        """This function should *only* be used inside a FormFieldsContainer
+        """Return the member widget named item.
+
+        This function should *only* be used inside a FormFieldsContainer
         template, really, else the path acrobatics will lead to unexpected
-        results."""
+        results.
+
+        """
         field = self.get_field_by_name(item, None)
         if field is None:
             raise ValueError("Field for '%s' not found." % item)
@@ -550,11 +583,9 @@ class FormFieldsContainer(CompoundInputWidget):
     def update_params(self, d):
         super(FormFieldsContainer, self).update_params(d)
         d["display_field_for"] = lambda f: self.display_field_for(f,
-                                                                  d["value_for"](f),
-                                                                  **d["params_for"](f))
+            d["value_for"](f), **d["params_for"](f))
         d["render_field_for"] = lambda f: self.display_field_for(f,
-                                                                 d["value_for"](f),
-                                                                 **d["params_for"](f))
+            d["value_for"](f), **d["params_for"](f))
         d["field_for"] = self._field_for
         visible_fields = []
         hidden_fields = []
@@ -570,12 +601,15 @@ class FormFieldsContainer(CompoundInputWidget):
         d["fields"] = visible_fields
         d["hidden_fields"] = hidden_fields
 
+
 class CompoundFormField(FormFieldsContainer, FormField):
     is_required = False
 
+
 class RepeatingFormField(RepeatingInputWidget, CompoundFormField):
     pass
 
+
 #############################################################################
 # Fields                                                                    #
 #############################################################################
@@ -596,9 +630,11 @@ class Label(FormField):
                             'the label tag'}
     attrs = {}
 
+
 class LabelDesc(CoreWD):
     for_widget = Label(default="Sample Label")
 
+
 class TextField(FormField):
     """A standard, single-line text field."""
 
@@ -617,12 +653,14 @@ class TextField(FormField):
                             ' the input tag'}
     attrs = {}
 
+
 class TextFieldDesc(CoreWD):
     name = "Text Field"
     for_widget = TextField("your_name",
                            default="Your Name Here",
                            attrs=dict(size="30"))
 
+
 class PasswordField(FormField):
     """A password field which masks letters with * characters."""
 
@@ -641,11 +679,13 @@ class PasswordField(FormField):
                             'the password input tag'}
     attrs = {}
 
+
 class PasswordFieldDesc(CoreWD):
     name = "Password Field"
     for_widget = PasswordField("your_secret",
                                default="Top Secret Password")
 
+
 class HiddenField(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -663,10 +703,12 @@ class HiddenField(FormField):
     attrs = {}
     hidden = True
 
+
 class HiddenFieldDesc(CoreWD, RenderOnlyWD):
     name = "Hidden Field"
     for_widget = HiddenField("hidden_one")
 
+
 class FileField(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -686,10 +728,12 @@ class FileField(FormField):
     def display(self, value=None, **params):
         return super(FileField, self).display(None, **params)
 
+
 class FileFieldDesc(CoreWD):
     name = "File Field"
     for_widget = FileField("your_filefield", attrs=dict(size="30"))
 
+
 class Button(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -710,10 +754,12 @@ class Button(FormField):
             d['attrs']['name'] = d["name"]
             d['attrs']['id'] = d["field_id"]
 
+
 class ButtonDesc(CoreWD):
     name = "Button"
     for_widget = Button(default="Button Value")
 
+
 class SubmitButton(Button):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -724,10 +770,12 @@ class SubmitButton(Button):
     />
     """
 
+
 class SubmitButtonDesc(CoreWD):
     name = "Submit Button"
     for_widget = SubmitButton(default="Submit Button Value")
 
+
 class ResetButton(Button):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -738,10 +786,12 @@ class ResetButton(Button):
     />
     """
 
+
 class ResetButtonDesc(CoreWD):
     name = "Reset Button"
     for_widget = ResetButton(default="Reset Button Value")
 
+
 class ImageButton(Button):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -765,10 +815,12 @@ class ImageButton(Button):
     height = None
     alt = None
 
+
 class ImageButtonDesc(CoreWD):
     name = "Image Button"
     for_widget = ImageButton("your_image_button", src="/tg_static/images/toolbox_logo.png")
 
+
 class CheckBox(FormField):
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -800,6 +852,7 @@ class CheckBox(FormField):
         if value:
             d['attrs']['checked'] = 'checked'
 
+
 class CheckBoxDesc(CoreWD):
     for_widget = CheckBox(name="your_checkbox",
                           default=True,
@@ -813,6 +866,7 @@ class CheckBoxDesc(CoreWD):
     </div>
     """
 
+
 class TextArea(FormField):
     template = """
     <textarea xmlns:py="http://purl.org/kid/ns#"
@@ -834,12 +888,14 @@ class TextArea(FormField):
     cols = 50
     attrs = {}
 
+
 class TextAreaDesc(CoreWD):
     name = "Text Area"
     for_widget = TextArea(name="your_textarea",
                           default="Your Comment Here",
                           rows=5, cols=40)
 
+
 class SelectionField(FormField):
     # an empty selection doesn't submit anything
     no_input_if_empty = True
@@ -854,37 +910,30 @@ class SelectionField(FormField):
     def __init__(self, *args, **kw):
         super(SelectionField, self).__init__(*args, **kw)
         if not self.validator:
-            validator = self._guess_validator()
-            if validator is not None:
-                self.validator = validator
-            else:
-                raise ValueError, ("No validator specified and couldn't "
-                                   "guess one. I cannot work properly "
-                                   "without it")
+            try:
+                self.validator = self._guess_validator()
+            except Exception, err:
+                raise ValueError(
+                    "No validator specified and couldn't guess one:\n%s\n"
+                    "Please explicitly specify a validator for the %s."
+                    % (err, self.__class__.__name__))
         # Only override the user-provided validator if it's not a ForEach one,
         # which usually means the user needs to perform validation on the list
         # as a whole.
-        if self._multiple_selection and \
-           not isinstance(self.validator, validators.ForEach):
+        if (self._multiple_selection and
+                not isinstance(self.validator, validators.ForEach)):
             self.validator = validators.MultipleSelection(self.validator)
 
     def _guess_validator(self):
-        """
-        Inspect sample option value to guess validator. crude
-        avoid multiple evalution if options is callable
-        """
-        validator = None
+        """Inspect sample option value to guess validator (crude)."""
         sample_option = self._get_sample_option()
-        if sample_option is not None:
-            if isinstance(sample_option, int):
-                validator = validators.Int()
-            elif isinstance(sample_option, basestring):
-                validator = validators.String()
-            else:
-                import warnings
-                warnings.warn("unknown option type in SelectionField: " + \
-                       repr(sample_option[0]))
-        return validator
+        if isinstance(sample_option, int):
+            return validators.Int()
+        elif isinstance(sample_option, basestring):
+            return validators.String()
+        else:
+            raise TypeError("Unknown option type in SelectionField: %r"
+                % (sample_option,))
 
     def _get_sample_option(self):
         options = self._extend_options(self.options)
@@ -950,6 +999,7 @@ class SelectionField(FormField):
                     return True
         return False
 
+
 class SingleSelectField(SelectionField):
     template = """
     <select xmlns:py="http://purl.org/kid/ns#"
@@ -976,6 +1026,7 @@ class SingleSelectField(SelectionField):
                             ' the select tag'}
     attrs = {}
 
+
 class SingleSelectFieldDesc(CoreWD):
     name = "Single Select Field"
     for_widget = SingleSelectField("your_single_select_field",
@@ -985,6 +1036,7 @@ class SingleSelectFieldDesc(CoreWD):
                                             (4, "Ruby")],
                                    default=2)
 
+
 class MultipleSelectField(SelectionField):
     template = """
     <select xmlns:py="http://purl.org/kid/ns#"
@@ -1014,6 +1066,7 @@ class MultipleSelectField(SelectionField):
     attrs = {}
     size = 5
 
+
 class MultipleSelectFieldDesc(CoreWD):
     name = "Multiple Select Field"
     for_widget = MultipleSelectField("your_multiple_select_field",
@@ -1023,6 +1076,7 @@ class MultipleSelectFieldDesc(CoreWD):
                                               ("d", "Ruby")],
                                      default=["a","c","d"])
 
+
 class RadioButtonList(SelectionField):
     template = """
     <ul xmlns:py="http://purl.org/kid/ns#"
@@ -1046,11 +1100,13 @@ class RadioButtonList(SelectionField):
     params_doc = {'list_attrs' : 'Extra (X)HTML attributes for the ul tag'}
     list_attrs = {}
 
+
 class RadioButtonListDesc(CoreWD):
     name = "RadioButton List"
     for_widget = RadioButtonList("your_radiobutton_list",
         options=list(enumerate("Python Java Pascal Ruby".split())), default=3)
 
+
 class CheckBoxList(SelectionField):
     template = """
     <ul xmlns:py="http://purl.org/kid/ns#"
@@ -1075,12 +1131,14 @@ class CheckBoxList(SelectionField):
     params_doc = {'list_attrs' : 'Extra (X)HTML attributes for the ul tag'}
     list_attrs = {}
 
+
 class CheckBoxListDesc(CoreWD):
     name = "CheckBox List"
     for_widget = CheckBoxList("your_checkbox_list",
         options=list(enumerate("Python Java Pascal Ruby".split())),
         default=[0,3])
 
+
 class FieldSet(CompoundFormField):
     template="""
     <fieldset xmlns:py="http://purl.org/kid/ns#"
@@ -1103,6 +1161,7 @@ class FieldSet(CompoundFormField):
     params_doc = {'legend' : 'Text to display as the legend for the fieldset'}
     legend = None
 
+
 class FieldSetDesc(CoreWD):
     name = "FieldSet"
     for_widget = FieldSet("your_fieldset",
@@ -1112,6 +1171,7 @@ class FieldSetDesc(CoreWD):
                                   TextField(name="upper_limit",
                                             label="Upper Limit")])
 
+
 class RepeatingFieldSet(RepeatingFormField):
     template="""
     <div xmlns:py="http://purl.org/kid/ns#">
@@ -1136,6 +1196,7 @@ class RepeatingFieldSet(RepeatingFormField):
     params_doc = {'legend' : 'Text to display as the legend for the fieldset'}
     legend = None
 
+
 class RepeatingFieldSetDesc(CoreWD):
     name = "Repeating FieldSet"
     for_widget = RepeatingFieldSet("your_repeating_fieldset",
@@ -1146,6 +1207,7 @@ class RepeatingFieldSetDesc(CoreWD):
                                            TextField(name="upper_limit",
                                                      label="Upper Limit")])
 
+
 #############################################################################
 # Forms                                                                     #
 #############################################################################
@@ -1162,8 +1224,8 @@ class Form(FormFieldsContainer):
         'submit_text': 'Text for the submit button',
         'disabled_fields': 'List of names of the fields we want to disable'}
     submit = SubmitButton()
-    action = None
-    method = "post"
+    action = ''
+    method = 'post'
     form_attrs = {}
     use_name = True
     submit_text = None
@@ -1183,6 +1245,7 @@ class Form(FormFieldsContainer):
         return getattr(request, 'validation_errors', None)
     errors = property(_get_errors)
 
+
 class TableForm(Form):
     """Form with simple table layout."""
 
@@ -1223,6 +1286,7 @@ class TableForm(Form):
     params_doc = {'table_attrs' : 'Extra (X)HTML attributes for the Table tag'}
     table_attrs = {}
 
+
 class TableFormDesc(CoreWD):
     name = "Table Form"
     full_class_name = "turbogears.widgets.TableForm"
@@ -1246,6 +1310,7 @@ class TableFormDesc(CoreWD):
     def save(self, **kw):
         return """Received data from TableForm:<br />%r""" % kw
 
+
 class ListForm(Form):
     """Form with simple list layout."""
 
@@ -1279,6 +1344,7 @@ class ListForm(Form):
     params_doc = {'list_attrs' : 'Extra (X)HTML attributes for the ul tag'}
     list_attrs = {}
 
+
 class ListFormDesc(CoreWD):
     name = "List Form"
     full_class_name = "turbogears.widgets.ListForm"
